function replacePX(str) {
    if (str) {
        let reg = /px/
        return +str.replace(reg, "")
    } else {
        return 14
    }
}

function detectType(source) {
    return Object.prototype.toString
        .call(source)
        .split(/[\[,\s,\]]/)[2]
        .toLowerCase()
}

function deepCopy(source, copyDeep) {
    var type = detectType(source)
    if (!(type === "object" || type === "array")) {
        return source
    }
    var newObject = type === "array" ? source.slice() : Object.assign({}, source)
    if (!copyDeep) {
        return newObject
    }
    Object.keys(newObject).forEach(function (key) {
        newObject[key] = deepCopy(newObject[key], true)
    })
    return newObject
}

function renderText(el, d, isWidget, theme) {
    let fontSize = replacePX(d.fontSize)
    let textWidth = fontSize * (d.label ? d.label.length : 0)
    let fill = d.fill || "#444444"
    // 深色主题，设置字体颜色为白色
    if (theme === 'dark') {
        fill = '#fff'
    }
    let text = el.append("text")
    if (!isWidget) {
        text.attr("class", "element")
    }
    text.attr("x", d.x)
        .attr("y", d.y)
        .attr("text-anchor", d.anchor || "start")
        .attr("font-size", d.fontSize || "14px")
        .attr("font-weight", d.fontWeight || "normal")
        .attr("fill", fill)
        .text(d.label || "-")

    if (d.id) {
        text.attr("id", d.id)
    }
    // 文字书写方向
    if (d.writingMode) {
        text.attr("writing-mode", d.writingMode || "tb")
    }

    if (d.rotate) {
        if (!d.anchor || d.anchor === "start") {
            text.attr(
                "transform",
                `rotate(${d.rotate}, ${+d.x + textWidth / 2}, ${+d.y -
                    fontSize / 2})`
            )
        } else {
            text.attr(
                "transform",
                `rotate(${d.rotate}, ${d.x}, ${+d.y - fontSize / 2})`
            )
        }
    }
}

function renderImage(el, d, isWidget) {
    let image = el.append("image")
    if (!isWidget) {
        image.attr("class", "element")
    }
    image
        .attr("x", d.x)
        .attr("y", d.y)
        .attr("width", d.width)
        .attr("height", d.height)
        .attr("xlink:href", d.url)
        .attr("cursor", d.cursor ||
            "default")
    if (d.rotate) {
        image.attr(
            "transform",
            `rotate(${d.rotate}, ${+d.x + d.width / 2}, ${+d.y +
                d.height / 2})`
        )
    }
    if (d.isAnimate && d.value) {
        let anime = image.append("animate")
        if (!d.anime && Object.keys(d.anime).length > 0) {
            return
        }
        anime.attr("attributeName", d.anime.direction || "x")

            .attr("dur", d.anime.duration || "2s")
            .attr("repeatCount", "indefinite")

        if (d.anime.direction === "y") {
            anime.attr("from", d.y)
            anime.attr("to", +d.y + (+d.anime.distance || 0))
        } else {
            anime.attr("from", d.x)
            anime.attr("to", +d.x + (+d.anime.distance || 0))
        }
    }
}

function renderCircle(el, d, isWidget) {
    let circle = el.append("circle")
    if (!isWidget) {
        circle.attr("class", "element")
    }
    circle
        .attr("cx", d.cx)
        .attr("cy", d.cy)
        .attr("r", d.r)
        .attr("fill", d.fill)

    if (d.opacity) {
        circle.style("opacity", d.opacity)
    }
}

function renderRect(el, d, isWidget) {
    let rect = el.append("rect")
    if (!isWidget) {
        rect.attr("class", "element")
    }
    rect.attr("x", d.x)
        .attr("y", d.y)
        .attr("width", d.width)
        .attr("height", d.height)
        .attr("fill", d.fill)
        .attr("stroke", d.stroke || d.fill)
        .attr("stroke-width", d.strokeWidth || 0)

    if (d.opacity) {
        rect.style("opacity", d.opacity)
    }
}

function renderBelt(el, d, theme) {
    let group = el.append("g").attr("class", "element")
    // 绘制皮带
    for (let i = 0; i < d.size; i++) {
        let obj = Object.assign({}, d)
        obj.x = +obj.x + i * obj.width || 0
        obj.rotate = 0
        renderImage(group, obj, true)
    }
    // 绘制煤流
    let coalPattern = "#coal"
    if (d.reverse) {
        coalPattern = "#coal-reverse"
    }

    if (d.value) {
        let obj = Object.assign({}, d)
        obj.width = obj.width * obj.size
        obj.height = 10
        obj.fill = `url(${coalPattern})`
        renderRect(group, obj, true)
    }
    // 绘制文字
    if (d.label) {
        let obj = Object.assign({}, d)
        obj.x = +obj.x + obj.width * obj.size / 2
        obj.y = obj.y + obj.height
        obj.anchor = "middle"
        renderText(group, obj, true, theme)
    }

    // 旋转
    if (d.rotate) {
        group.attr(
            "transform",
            `rotate(${d.rotate}, ${d.x +
                            (d.width * d.size) / 2}, ${d.y +
                            d.height / 2})`
        )
    }
}

function renderSwitch(el, d, theme) {
    let group = el.append("g").attr("class", "element")
    renderImage(group, d, true)

    if (d.textTop) {
        let textTopObj = Object.assign({}, d)
        textTopObj.id = "#text-top"
        textTopObj.label = d.textTop
        textTopObj.x = +d.x + d.width / 2
        textTopObj.y = d.y - 20
        textTopObj.anchor = "middle"
        textTopObj.rotate = 0
        renderText(group, textTopObj, true, theme)
    }

    if (d.textMid) {
        let textMidObj = Object.assign({}, d)
        textMidObj.id = "#text-mid"
        textMidObj.label = d.textMid
        textMidObj.x = +d.x + d.width / 2
        textMidObj.y = d.y - 5
        textMidObj.anchor = "middle"
        textMidObj.rotate = 0
        renderText(group, textMidObj, true, theme)
    }

    if (d.textBottom) {
        let isLongText = false
        let textStart = d.textBottom
        let textEnd = ""

        if (d.textBottom.split("").length > 9) {
            textStart = d.textBottom.slice(0, 9)
            textEnd = d.textBottom.slice(9)
            isLongText = true
        }

        let textBottomObj = Object.assign({}, d)

        textBottomObj.id = "#text-bottom"
        textBottomObj.label = textStart
        textBottomObj.x = d.x + 34
        textBottomObj.y = d.y + 59
        textBottomObj.writingMode = "tb"
        textBottomObj.rotate = 0

        renderText(group, textBottomObj, true, theme)

        if (isLongText) {
            let textBottomMoreObj = Object.assign({}, d)

            textBottomMoreObj.id = "#text-bottom-more"
            textBottomMoreObj.label = textEnd
            textBottomMoreObj.x = d.x + 14
            textBottomMoreObj.y = d.y + 59
            textBottomMoreObj.writingMode = "tb"
            textBottomMoreObj.rotate = 0
            renderText(group, textBottomMoreObj, true, theme)
        }
    }

    // 旋转s
    if (d.rotate) {
        group.attr(
            "transform",
            `rotate(${d.rotate}, ${d.x +
                      d.width / 2}, ${d.y + d.height / 2})`
        )
    }
}

function renderVideo(el, d) {
    let group = el.append("g").attr("class", "element")
    // 渲染主图时，不绘制video
    if (!d.isHidden) {
        renderRect(group, d, true)
    }

}

function renderTextGroup(el, d, theme) {
    let group = el.append("g").attr("class", "element")
    let texts = d.texts ? d.texts.split(",") : []

    texts.forEach((item, idx) => {
        let target = Object.assign({}, d)
        let direction = target.direction || "x"
        let reg = /\$\(([^]+?)\)/g
        if (direction === "x") {
            target.x = +target.x + target.space * idx
        } else if (direction === "y") {
            target.y = +target.y + target.space * idx
        }
        if (reg.test(item)) {
            item = item.replace(reg, "0.00")
        }
        target.label = item
        renderText(group, target, true, theme)
    })
}

export {
    renderText,
    renderImage,
    renderCircle,
    renderRect,
    renderBelt,
    renderSwitch,
    renderVideo,
    renderTextGroup,
    replacePX
}